///////////////////////////////////////////////////////////////////////////
////                                                                   ////
////                 Driver for Sweeden LCD modules                    ////
////     void lcd_init(BYTE display_on_off_ctrl, BYTE entry_mode)      ////
////     printf(lcd_putc,"string goes here",pramerters)                ////
////     lcd_char_generator(BYTE charnumber, BYTE *pattern)            ////
////     lcd_cursor_goto(BYTE row,col)                                 ////
///////////////////////////////////////////////////////////////////////////


// Pins
#ifndef LCD_RS
#define LCD_RS        PIN_B5                  // Data/Control pin
#endif
#ifndef LCD_E
#define LCD_E         PIN_B4                  // latch pin
#endif
#ifndef LCD_D7
#define LCD_D7        PIN_B3                  // data pin D7
#endif
#ifndef LCD_D6
#define LCD_D6        PIN_B2                  // data pin D6
#endif
#ifndef LCD_D5
#define LCD_D5        PIN_B1                  // data pin D5
#endif
#ifndef LCD_D4
#define LCD_D4        PIN_B0                  // data pin D4
#endif

#define CTRLBYTE      true

//display_on_off_ctrl
#define DISPLAY_ON    4
#define CURSOR_ON     2
#define CURSOR_BLINK  1

//entry_mode
#define CURSOR_DIR_UP  2
#define DISPLAY_SHIFT 1

#define lcd_line_two 0x40    // LCD RAM address for the second line

//macros functions
#define lcd_cursor_gohome()            lcd_send_control(2)
#define lcd_cursor_left()              lcd_send_control(16)
#define lcd_cursor_right()             lcd_send_control(20)
#define lcd_cursor_underline()         lcd_send_control(14)
#define lcd_cursor_blinking()          lcd_send_control(15)
#define lcd_cursor_off()               lcd_send_control(12)
#define lcd_display_clr()              lcd_send_control(1)
#define lcd_display_scrollleft()       lcd_send_control(24)
#define lcd_display_scrollright()      lcd_send_control(28)
#define lcd_display_blank()            lcd_send_control(8)
#define lcd_display_restore()          lcd_send_control(12)

BYTE ac;
boolean cursordirup;

void lcd_send_nibble( BYTE n ) {
//load data
if(bit_test(n,3)) output_high(LCD_D7); else output_low(LCD_D7);
if(bit_test(n,2)) output_high(LCD_D6); else output_low(LCD_D6);
if(bit_test(n,1)) output_high(LCD_D5); else output_low(LCD_D5);
if(bit_test(n,0)) output_high(LCD_D4); else output_low(LCD_D4);
//latch data
delay_cycles(1);
output_high(LCD_E);
delay_us(2);
output_low(LCD_E);
delay_us(2);
}

void lcd_send_byte(  BYTE n ) {
lcd_send_nibble(n >> 4);
lcd_send_nibble(n & 0xf);
delay_us(45);
if(n<=3) delay_ms(2);
}

void lcd_send_control(BYTE n){
output_low(LCD_RS);
lcd_send_byte(n);
}

void lcd_send_char(BYTE n){
output_high(LCD_RS);
lcd_send_byte(n);
}

void lcd_cursor_goto(BYTE row,col){
ac=(lcd_line_two*(row-1))|(col-1);
lcd_send_control(128|ac);
}

void lcd_putc(BYTE c){
switch (c) {
     case '\f'   : lcd_display_clr();
                  break;
     case '\n'   : lcd_cursor_goto(2,1);
                  break;
     case '\b'   : lcd_cursor_left();
                  break;
     default     : lcd_send_char(c);
                  break;
   }
if(cursordirup) ac++; else ac--;
}

void lcd_char_generator(BYTE charnumber, BYTE *pattern){
int i;
lcd_send_control(64|(charnumber*8)); // set CG address to write to
for(i=0;i<=7;i++){
lcd_send_char(pattern[i]); //not a char but the five bits of the pattern
}
lcd_send_control(128|ac); // set the ac pointer to DD ram
}

void lcd_init(BYTE display_on_off_ctrl, BYTE entry_mode){
BYTE i;
if(bit_test(entry_mode,CURSOR_DIR_UP)) cursordirup=true; else cursordirup=false;
output_low(LCD_E);
output_low(LCD_RS);
delay_ms(60);

//set 8bit mode  (to make sure we are in sync with 4bit nibbles)
for(i=1;i<=3;++i) {
lcd_send_nibble(3);
delay_ms(5);
}
lcd_send_nibble(2); // set 4bit mode ? rows
delay_us(50);

lcd_send_control(40);               // set 4 bit mode 2 rows
lcd_send_control(8|display_on_off_ctrl);
lcd_display_clr();                                // clear screen
lcd_send_control(4|entry_mode);
lcd_cursor_gohome();ac=0;                // set adress counter to zero (ac=0)
}

